
#include "nrf_esb.h"
#include <stdbool.h>
#include <stdint.h>
#include <string.h>
#include "sdk_common.h"
#include "nrf.h"
#include "nrf_esb_error_codes.h"
#include "nrf_delay.h"
#include "nrf_gpio.h"
#include "nrf_error.h"
#include "nrf_log.h"
#include "boards.h"
#include "nrf_log.h"
#include "app_uart.h"
#include "app_error.h"


#define LED_ON          0
#define LED_OFF         1
#define ENABLE_LOOPBACK_TEST  /**< if defined, then this example will be a loopback test, which means that TX should be connected to RX to get data loopback. */
#define MAX_TEST_DATA_BYTES     (15U)                /**< max number of test bytes to be used for tx and rx. */
#define UART_TX_BUF_SIZE 256                         /**< UART TX buffer size. */
#define UART_RX_BUF_SIZE 1                           /**< UART RX buffer size. */


void nrf_esb_error_handler(uint32_t err_code, uint32_t line)
{
#if DEBUG //lint -e553
    while(true);
#else
    NVIC_SystemReset();
#endif

}

//#define APP_ERROR_CHECK(err_code) if(err_code) nrf_esb_error_handler(err_code, __LINE__);

//#define NRF_ESB_LEGACY

/*lint -save -esym(40, BUTTON_1) -esym(40, BUTTON_2) -esym(40, BUTTON_3) -esym(40, BUTTON_4) -esym(40, LED_1) -esym(40, LED_2) -esym(40, LED_3) -esym(40, LED_4) */

static nrf_esb_payload_t tx_payload;
static nrf_esb_payload_t rx_payload;
//static uint8_t m_state[4];
const uint8_t leds_list[LEDS_NUMBER] = LEDS_LIST;
uint8_t led_nr;

// void nrf_esb_event_handler(nrf_esb_evt_t const * p_event)
// {
    // switch (p_event->evt_id)
    // {
        // case NRF_ESB_EVENT_TX_SUCCESS:
            // NRF_LOG("SUCCESS\r\n");
            // break;
        // case NRF_ESB_EVENT_TX_FAILED:
            // NRF_LOG("FAILED\r\n");
            // (void) nrf_esb_flush_tx();
            // break;
        // case NRF_ESB_EVENT_RX_RECEIVED:
            // while (nrf_esb_read_rx_payload(&rx_payload) == NRF_SUCCESS) ;
            // NRF_LOG("Receiving packet: ");
            // //NRF_LOG_HEX_CHAR(rx_payload.data[0]);
			// NRF_LOG_PRINTF("Packet = %c  \r\n",rx_payload.data[0]);
            // NRF_LOG("\r\n");

            // switch (rx_payload.data[0] & 0xFUL)
            // {
                // case 0x1:
                    // m_state[0] = !m_state[0];
                    // nrf_gpio_pin_write(LED_1, m_state[0]);
                    // break;

                // case 0x2:
                    // m_state[1] = !m_state[1];
                    // nrf_gpio_pin_write(LED_2, m_state[1]);
                    // break;

                // case 0x4:
                    // m_state[2] = !m_state[2];
                    // nrf_gpio_pin_write(LED_3, m_state[2]);
                    // break;

                // case 0x8:
                    // m_state[3] = !m_state[3];
                    // nrf_gpio_pin_write(LED_4, m_state[3]);
                    // break;
            // }

            // nrf_gpio_pin_write(LED_1, m_state[0]);
            // nrf_gpio_pin_write(LED_2, m_state[1]);
            // nrf_gpio_pin_write(LED_3, m_state[2]);
            // nrf_gpio_pin_write(LED_4, m_state[3]);

            // tx_payload.length = 1;
            // tx_payload.data[0] = m_state[0] << 0
                               // | m_state[1] << 1
                               // | m_state[2] << 2
                               // | m_state[3] << 3;
            // (void) nrf_esb_write_payload(&tx_payload);

            // NRF_LOG("Queue transmitt packet: ");
            // NRF_LOG_HEX_CHAR(tx_payload.data[0]);
            // NRF_LOG("\r\n");
            // break;
    // }



// }


void clocks_start( void )
{
    // Start HFCLK and wait for it to start.
    NRF_CLOCK->EVENTS_HFCLKSTARTED = 0;
    NRF_CLOCK->TASKS_HFCLKSTART = 1;
    while (NRF_CLOCK->EVENTS_HFCLKSTARTED == 0);
}


uint32_t esb_init( void )
{
    uint32_t err_code;
    uint8_t base_addr_0[4] = {0xE7, 0xE7, 0xE7, 0xE7};
    uint8_t base_addr_1[4] = {0xC2, 0xC2, 0xC2, 0xC2};
    uint8_t addr_prefix[8] = {0xE7, 0xC2, 0xC3, 0xC4, 0xC5, 0xC6, 0xC7, 0xC8 };

#ifndef NRF_ESB_LEGACY
    nrf_esb_config_t nrf_esb_config         = NRF_ESB_DEFAULT_CONFIG;
#else // NRF_ESB_LEGACY
    nrf_esb_config_t nrf_esb_config         = NRF_ESB_LEGACY_CONFIG;
#endif // NRF_ESB_LEGACY
    nrf_esb_config.selective_auto_ack       = 0;
    nrf_esb_config.payload_length           = 3;
    nrf_esb_config.bitrate                  = NRF_ESB_BITRATE_2MBPS;
    nrf_esb_config.mode                     = NRF_ESB_MODE_PRX;
    //nrf_esb_config.event_handler            = nrf_esb_event_handler;

    err_code = nrf_esb_init(&nrf_esb_config);
    VERIFY_SUCCESS(err_code);

    err_code = nrf_esb_set_base_address_0(base_addr_0);
    VERIFY_SUCCESS(err_code);

    err_code = nrf_esb_set_base_address_1(base_addr_1);
    VERIFY_SUCCESS(err_code);

    err_code = nrf_esb_set_prefixes(addr_prefix, 8);
    VERIFY_SUCCESS(err_code);

    tx_payload.length = 1;

    return NRF_SUCCESS;
}

// void gpio_init( void )
// {
    // m_state[0] = LED_OFF;
    // m_state[1] = LED_OFF;
    // m_state[2] = LED_OFF;
    // m_state[3] = LED_OFF;

    // //nrf_gpio_range_cfg_output(8, 31);
    // LEDS_CONFIGURE(LEDS_MASK);

    // nrf_gpio_pin_write(LED_1, m_state[0]);
    // nrf_gpio_pin_write(LED_2, m_state[1]);
    // nrf_gpio_pin_write(LED_3, m_state[2]);
    // nrf_gpio_pin_write(LED_4, m_state[3]);
// }


uint32_t logging_init( void )
{
    uint32_t err_code;
    err_code = NRF_LOG_INIT();
    return err_code;
}


void power_manage( void )
{
    // WFE - SEV - WFE sequence to wait until a radio event require further actions.
    __WFE();
    __SEV();
    __WFE();
}

void uart_error_handle(app_uart_evt_t * p_event)
{
    if (p_event->evt_type == APP_UART_COMMUNICATION_ERROR)
    {
        APP_ERROR_HANDLER(p_event->data.error_communication);
    }
    else if (p_event->evt_type == APP_UART_FIFO_ERROR)
    {
        APP_ERROR_HANDLER(p_event->data.error_code);
    }
}

// void uart_loopback_test()
// {
    // // uint8_t * tx_data = (uint8_t *)("\n\rLOOPBACK_TEST\n\r");
    // // uint8_t   rx_data;

	// char arr[14] = "KAIMSOFTECH  ";
	// int i;
    // // Start sending one byte and see if you get the same
    // for (i = 0; i < 14; i++)
    // {
        // //uint32_t err_code;
        // //while(app_uart_put('H') != NRF_SUCCESS);
		// //while(app_uart_put('a'));
		// //while(app_uart_put('b'));
		// //while(app_uart_put('c'));
 
  // while(app_uart_put(arr[i]) != NRF_SUCCESS);
		
        // nrf_delay_ms(1000);
        
    // }
	// //printf("\n");
	// printf("\r\t\t");
    // return;
// }

int main(void)
{
    uint32_t err_code;
    // err_code = logging_init();
    // APP_ERROR_CHECK(err_code);
    //gpio_init();
    err_code = esb_init();
    APP_ERROR_CHECK(err_code);
    clocks_start();

    //NRF_LOG("Enhanced ShockBurst Receiver Example running.\r\n");

    err_code = nrf_esb_start_rx();
    APP_ERROR_CHECK(err_code);

    // tx_payload.data[0] = m_state[0] << 0
                       // | m_state[1] << 1
                       // | m_state[2] << 2
                       // | m_state[3] << 3;
    // err_code = nrf_esb_write_payload(&tx_payload);
    // APP_ERROR_CHECK(err_code);
	LEDS_CONFIGURE(LEDS_MASK);
    LEDS_OFF(LEDS_MASK);
    //uint32_t err_code;
    const app_uart_comm_params_t comm_params =
      {
          RX_PIN_NUMBER,
          TX_PIN_NUMBER,
          RTS_PIN_NUMBER,
          CTS_PIN_NUMBER,
          APP_UART_FLOW_CONTROL_DISABLED,
          false,
          UART_BAUDRATE_BAUDRATE_Baud115200
      };

    APP_UART_FIFO_INIT(&comm_params,
                         UART_RX_BUF_SIZE,
                         UART_TX_BUF_SIZE,
                         uart_error_handle,
                         APP_IRQ_PRIORITY_LOW,
                         err_code);

    APP_ERROR_CHECK(err_code);

    while (true)
    {
		nrf_gpio_pin_dir_set(23, 1);
		nrf_gpio_pin_set(23);
        //uart_loopback_test();
        //power_manage();
		
		while (nrf_esb_read_rx_payload(&rx_payload) == NRF_SUCCESS) ;
		while(app_uart_put(((char)rx_payload.rssi)) != NRF_SUCCESS);
            for(int i=0; i<1; i++)
			{
			//NRF_LOG("Receiving packet: ");
            //NRF_LOG_HEX_CHAR(rx_payload.data[0]);
			while(app_uart_put(rx_payload.data[0]) != NRF_SUCCESS);
			//nrf_delay_ms(10);
			while(app_uart_put(rx_payload.data[1]) != NRF_SUCCESS);
			//nrf_delay_ms(10);
			while(app_uart_put(rx_payload.data[2]) != NRF_SUCCESS);
			}
			
			while (nrf_esb_read_rx_payload(&rx_payload) != NRF_SUCCESS) ;
			nrf_esb_flush_rx() ;
			//nrf_delay_ms(10);
			//while(app_uart_put('') != NRF_SUCCESS);
			//printf("BHA \r\t\t");
			//printf("\r\t\t");
			//NRF_LOG_PRINTF("Address = 0x%c%c Temperature = %c  \r\n",rx_payload.data[0],rx_payload.data[1],rx_payload.data[2]);
    }
}
/*lint -restore */


